@@ -1,6 +1,6 @@
 /* Target-dependent code for FreeBSD, architecture-independent.
 
-   Copyright (C) 2002-2022 Free Software Foundation, Inc.
+   Copyright (C) 2002-2025 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,15 +17,15 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "auxv.h"
+#include "exceptions.h"
+#include "extract-store-integer.h"
 #include "gdbcore.h"
 #include "inferior.h"
 #include "objfiles.h"
 #include "regcache.h"
 #include "regset.h"
 #include "gdbthread.h"
-#include "objfiles.h"
 #include "xml-syscall.h"
 #include <sys/socket.h>
 #include <arpa/inet.h>
@@ -33,6 +33,7 @@
 #include "elf-bfd.h"
 #include "fbsd-tdep.h"
 #include "gcore-elf.h"
+#include "arch-utils.h"
 
 /* This enum is derived from FreeBSD's <sys/signal.h>.  */
 
@@ -563,16 +564,16 @@ fbsd_core_thread_name (struct gdbarch *gdbarch, struct thread_info *thr)
 	 extract the null-terminated name from the start of the
 	 note.  */
       thread_section_name section_name (".thrmisc", thr->ptid);
+      bfd *cbfd = current_program_space->core_bfd ();
 
-      section = bfd_get_section_by_name (core_bfd, section_name.c_str ());
+      section = bfd_get_section_by_name (cbfd, section_name.c_str ());
       if (section != NULL && bfd_section_size (section) > 0)
 	{
 	  /* Truncate the name if it is longer than "buf".  */
 	  size = bfd_section_size (section);
 	  if (size > sizeof buf - 1)
 	    size = sizeof buf - 1;
-	  if (bfd_get_section_contents (core_bfd, section, buf, (file_ptr) 0,
-					size)
+	  if (bfd_get_section_contents (cbfd, section, buf, (file_ptr) 0, size)
 	      && buf[0] != '\0')
 	    {
 	      buf[size] = '\0';
@@ -581,7 +582,7 @@ fbsd_core_thread_name (struct gdbarch *gdbarch, struct thread_info *thr)
 		 as its thread name instead of an empty name if a name
 		 has not been set explicitly.  Return a NULL name in
 		 that case.  */
-	      if (strcmp (buf, elf_tdata (core_bfd)->core->program) != 0)
+	      if (strcmp (buf, elf_tdata (cbfd)->core->program) != 0)
 		return buf;
 	    }
 	}
@@ -606,12 +607,13 @@ fbsd_core_xfer_siginfo (struct gdbarch *gdbarch, gdb_byte *readbuf,
     return -1;
 
   thread_section_name section_name (".note.freebsdcore.lwpinfo", inferior_ptid);
-  asection *section = bfd_get_section_by_name (core_bfd, section_name.c_str ());
+  bfd *cbfd = current_program_space->core_bfd ();
+  asection *section = bfd_get_section_by_name (cbfd, section_name.c_str ());
   if (section == NULL)
     return -1;
 
   gdb_byte buf[4];
-  if (!bfd_get_section_contents (core_bfd, section, buf,
+  if (!bfd_get_section_contents (cbfd, section, buf,
 				 LWPINFO_OFFSET + LWPINFO_PL_FLAGS, 4))
     return -1;
 
@@ -628,21 +630,18 @@ fbsd_core_xfer_siginfo (struct gdbarch *gdbarch, gdb_byte *readbuf,
   else
     siginfo_offset = LWPINFO_OFFSET + LWPINFO64_PL_SIGINFO;
 
-  if (!bfd_get_section_contents (core_bfd, section, readbuf,
+  if (!bfd_get_section_contents (cbfd, section, readbuf,
 				 siginfo_offset + offset, len))
     return -1;
 
   return len;
 }
 
-static int
-find_signalled_thread (struct thread_info *info, void *data)
+static bool
+find_signalled_thread (struct thread_info *info)
 {
-  if (info->stop_signal () != GDB_SIGNAL_0
-      && info->ptid.pid () == inferior_ptid.pid ())
-    return 1;
-
-  return 0;
+  return (info->stop_signal () != GDB_SIGNAL_0
+	  && info->ptid.pid () == inferior_ptid.pid ());
 }
 
 /* Return a byte_vector containing the contents of a core dump note
@@ -650,10 +649,10 @@ find_signalled_thread (struct thread_info *info, void *data)
    the data is prefixed with a 32-bit integer size to match the format
    used in FreeBSD NT_PROCSTAT_* notes.  */
 
-static gdb::optional<gdb::byte_vector>
+static std::optional<gdb::byte_vector>
 fbsd_make_note_desc (enum target_object object, uint32_t structsize)
 {
-  gdb::optional<gdb::byte_vector> buf =
+  std::optional<gdb::byte_vector> buf =
     target_read_alloc (current_inferior ()->top_target (), object, NULL);
   if (!buf || buf->empty ())
     return {};
@@ -663,7 +662,7 @@ fbsd_make_note_desc (enum target_object object, uint32_t structsize)
 
   gdb::byte_vector desc (sizeof (structsize) + buf->size ());
   memcpy (desc.data (), &structsize, sizeof (structsize));
-  memcpy (desc.data () + sizeof (structsize), buf->data (), buf->size ());
+  std::copy (buf->begin (), buf->end (), desc.data () + sizeof (structsize));
   return desc;
 }
 
@@ -683,9 +682,9 @@ fbsd_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
 
   gdb_assert (gdbarch_iterate_over_regset_sections_p (gdbarch));
 
-  if (get_exec_file (0))
+  if (current_program_space->exec_filename () != nullptr)
     {
-      const char *fname = lbasename (get_exec_file (0));
+      const char *fname = lbasename (current_program_space->exec_filename ());
       std::string psargs = fname;
 
       const std::string &infargs = current_inferior ()->args ();
@@ -716,7 +715,7 @@ fbsd_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
     signalled_thr = curr_thr;
   else
     {
-      signalled_thr = iterate_over_threads (find_signalled_thread, NULL);
+      signalled_thr = iterate_over_threads (find_signalled_thread);
       if (signalled_thr == NULL)
 	signalled_thr = curr_thr;
     }
@@ -735,7 +734,7 @@ fbsd_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
 
   /* Auxiliary vector.  */
   uint32_t structsize = gdbarch_ptr_bit (gdbarch) / 4; /* Elf_Auxinfo  */
-  gdb::optional<gdb::byte_vector> note_desc =
+  std::optional<gdb::byte_vector> note_desc =
     fbsd_make_note_desc (TARGET_OBJECT_AUXV, structsize);
   if (note_desc && !note_desc->empty ())
     {
@@ -773,8 +772,12 @@ fbsd_make_corefile_notes (struct gdbarch *gdbarch, bfd *obfd, int *note_size)
 	return NULL;
     }
 
-  /* Include the target description when possible.  */
-  gcore_elf_make_tdesc_note (obfd, &note_data, note_size);
+  /* Include the target description when possible.  Some architectures
+     allow for per-thread gdbarch so we should really be emitting a tdesc
+     per-thread, however, we don't currently support reading in a
+     per-thread tdesc, so just emit the tdesc for the signalled thread.  */
+  gdbarch = target_thread_architecture (signalled_thr->ptid);
+  gcore_elf_make_tdesc_note (gdbarch, obfd, &note_data, note_size);
 
   return note_data;
 }
@@ -1017,8 +1020,8 @@ fbsd_info_proc_files_entry (int kf_type, int kf_fd, int kf_flags,
 static void
 fbsd_core_info_proc_files (struct gdbarch *gdbarch)
 {
-  asection *section
-    = bfd_get_section_by_name (core_bfd, ".note.freebsdcore.files");
+  bfd *cbfd = current_program_space->core_bfd ();
+  asection *section = bfd_get_section_by_name (cbfd, ".note.freebsdcore.files");
   if (section == NULL)
     {
       warning (_("unable to find open files in core file"));
@@ -1030,7 +1033,7 @@ fbsd_core_info_proc_files (struct gdbarch *gdbarch)
     error (_("malformed core note - too short for header"));
 
   gdb::def_vector<unsigned char> contents (note_size);
-  if (!bfd_get_section_contents (core_bfd, section, contents.data (),
+  if (!bfd_get_section_contents (cbfd, section, contents.data (),
 				 0, note_size))
     error (_("could not get core note contents"));
 
@@ -1044,20 +1047,18 @@ fbsd_core_info_proc_files (struct gdbarch *gdbarch)
 
   while (descdata + KF_PATH < descend)
     {
-      ULONGEST structsize = bfd_get_32 (core_bfd, descdata + KF_STRUCTSIZE);
+      ULONGEST structsize = bfd_get_32 (cbfd, descdata + KF_STRUCTSIZE);
       if (structsize < KF_PATH)
 	error (_("malformed core note - file structure too small"));
 
-      LONGEST type = bfd_get_signed_32 (core_bfd, descdata + KF_TYPE);
-      LONGEST fd = bfd_get_signed_32 (core_bfd, descdata + KF_FD);
-      LONGEST flags = bfd_get_signed_32 (core_bfd, descdata + KF_FLAGS);
-      LONGEST offset = bfd_get_signed_64 (core_bfd, descdata + KF_OFFSET);
-      LONGEST vnode_type = bfd_get_signed_32 (core_bfd,
-					      descdata + KF_VNODE_TYPE);
-      LONGEST sock_domain = bfd_get_signed_32 (core_bfd,
-					       descdata + KF_SOCK_DOMAIN);
-      LONGEST sock_type = bfd_get_signed_32 (core_bfd, descdata + KF_SOCK_TYPE);
-      LONGEST sock_protocol = bfd_get_signed_32 (core_bfd,
+      LONGEST type = bfd_get_signed_32 (cbfd, descdata + KF_TYPE);
+      LONGEST fd = bfd_get_signed_32 (cbfd, descdata + KF_FD);
+      LONGEST flags = bfd_get_signed_32 (cbfd, descdata + KF_FLAGS);
+      LONGEST offset = bfd_get_signed_64 (cbfd, descdata + KF_OFFSET);
+      LONGEST vnode_type = bfd_get_signed_32 (cbfd, descdata + KF_VNODE_TYPE);
+      LONGEST sock_domain = bfd_get_signed_32 (cbfd, descdata + KF_SOCK_DOMAIN);
+      LONGEST sock_type = bfd_get_signed_32 (cbfd, descdata + KF_SOCK_TYPE);
+      LONGEST sock_protocol = bfd_get_signed_32 (cbfd,
 						 descdata + KF_SOCK_PROTOCOL);
       fbsd_info_proc_files_entry (type, fd, flags, offset, vnode_type,
 				  sock_domain, sock_type, sock_protocol,
@@ -1150,8 +1151,9 @@ fbsd_core_info_proc_mappings (struct gdbarch *gdbarch)
   asection *section;
   unsigned char *descdata, *descend;
   size_t note_size;
+  bfd *cbfd = current_program_space->core_bfd ();
 
-  section = bfd_get_section_by_name (core_bfd, ".note.freebsdcore.vmmap");
+  section = bfd_get_section_by_name (cbfd, ".note.freebsdcore.vmmap");
   if (section == NULL)
     {
       warning (_("unable to find mappings in core file"));
@@ -1163,8 +1165,7 @@ fbsd_core_info_proc_mappings (struct gdbarch *gdbarch)
     error (_("malformed core note - too short for header"));
 
   gdb::def_vector<unsigned char> contents (note_size);
-  if (!bfd_get_section_contents (core_bfd, section, contents.data (),
-				 0, note_size))
+  if (!bfd_get_section_contents (cbfd, section, contents.data (), 0, note_size))
     error (_("could not get core note contents"));
 
   descdata = contents.data ();
@@ -1176,15 +1177,15 @@ fbsd_core_info_proc_mappings (struct gdbarch *gdbarch)
   fbsd_info_proc_mappings_header (gdbarch_addr_bit (gdbarch));
   while (descdata + KVE_PATH < descend)
     {
-      ULONGEST structsize = bfd_get_32 (core_bfd, descdata + KVE_STRUCTSIZE);
+      ULONGEST structsize = bfd_get_32 (cbfd, descdata + KVE_STRUCTSIZE);
       if (structsize < KVE_PATH)
 	error (_("malformed core note - vmmap entry too small"));
 
-      ULONGEST start = bfd_get_64 (core_bfd, descdata + KVE_START);
-      ULONGEST end = bfd_get_64 (core_bfd, descdata + KVE_END);
-      ULONGEST offset = bfd_get_64 (core_bfd, descdata + KVE_OFFSET);
-      LONGEST flags = bfd_get_signed_32 (core_bfd, descdata + KVE_FLAGS);
-      LONGEST prot = bfd_get_signed_32 (core_bfd, descdata + KVE_PROTECTION);
+      ULONGEST start = bfd_get_64 (cbfd, descdata + KVE_START);
+      ULONGEST end = bfd_get_64 (cbfd, descdata + KVE_END);
+      ULONGEST offset = bfd_get_64 (cbfd, descdata + KVE_OFFSET);
+      LONGEST flags = bfd_get_signed_32 (cbfd, descdata + KVE_FLAGS);
+      LONGEST prot = bfd_get_signed_32 (cbfd, descdata + KVE_PROTECTION);
       fbsd_info_proc_mappings_entry (gdbarch_addr_bit (gdbarch), start, end,
 				     offset, flags, prot, descdata + KVE_PATH);
 
@@ -1201,8 +1202,9 @@ fbsd_core_vnode_path (struct gdbarch *gdbarch, int fd)
   asection *section;
   unsigned char *descdata, *descend;
   size_t note_size;
+  bfd *cbfd = current_program_space->core_bfd ();
 
-  section = bfd_get_section_by_name (core_bfd, ".note.freebsdcore.files");
+  section = bfd_get_section_by_name (cbfd, ".note.freebsdcore.files");
   if (section == NULL)
     return nullptr;
 
@@ -1211,8 +1213,7 @@ fbsd_core_vnode_path (struct gdbarch *gdbarch, int fd)
     error (_("malformed core note - too short for header"));
 
   gdb::def_vector<unsigned char> contents (note_size);
-  if (!bfd_get_section_contents (core_bfd, section, contents.data (),
-				 0, note_size))
+  if (!bfd_get_section_contents (cbfd, section, contents.data (), 0, note_size))
     error (_("could not get core note contents"));
 
   descdata = contents.data ();
@@ -1225,12 +1226,12 @@ fbsd_core_vnode_path (struct gdbarch *gdbarch, int fd)
     {
       ULONGEST structsize;
 
-      structsize = bfd_get_32 (core_bfd, descdata + KF_STRUCTSIZE);
+      structsize = bfd_get_32 (cbfd, descdata + KF_STRUCTSIZE);
       if (structsize < KF_PATH)
 	error (_("malformed core note - file structure too small"));
 
-      if (bfd_get_32 (core_bfd, descdata + KF_TYPE) == KINFO_FILE_TYPE_VNODE
-	  && bfd_get_signed_32 (core_bfd, descdata + KF_FD) == fd)
+      if (bfd_get_32 (cbfd, descdata + KF_TYPE) == KINFO_FILE_TYPE_VNODE
+	  && bfd_get_signed_32 (cbfd, descdata + KF_FD) == fd)
 	{
 	  char *path = (char *) descdata + KF_PATH;
 	  return make_unique_xstrdup (path);
@@ -1247,20 +1248,22 @@ static void
 fbsd_core_fetch_timeval (struct gdbarch *gdbarch, unsigned char *data,
 			 LONGEST &sec, ULONGEST &usec)
 {
+  bfd *cbfd = current_program_space->core_bfd ();
+
   if (gdbarch_addr_bit (gdbarch) == 64)
     {
-      sec = bfd_get_signed_64 (core_bfd, data);
-      usec = bfd_get_64 (core_bfd, data + 8);
+      sec = bfd_get_signed_64 (cbfd, data);
+      usec = bfd_get_64 (cbfd, data + 8);
     }
-  else if (bfd_get_arch (core_bfd) == bfd_arch_i386)
+  else if (bfd_get_arch (cbfd) == bfd_arch_i386)
     {
-      sec = bfd_get_signed_32 (core_bfd, data);
-      usec = bfd_get_32 (core_bfd, data + 4);
+      sec = bfd_get_signed_32 (cbfd, data);
+      usec = bfd_get_32 (cbfd, data + 4);
     }
   else
     {
-      sec = bfd_get_signed_64 (core_bfd, data);
-      usec = bfd_get_32 (core_bfd, data + 8);
+      sec = bfd_get_signed_64 (cbfd, data);
+      usec = bfd_get_32 (cbfd, data + 8);
     }
 }
 
@@ -1269,10 +1272,11 @@ fbsd_core_fetch_timeval (struct gdbarch *gdbarch, unsigned char *data,
 static void
 fbsd_print_sigset (const char *descr, unsigned char *sigset)
 {
+  bfd *cbfd = current_program_space->core_bfd ();
   gdb_printf ("%s: ", descr);
   for (int i = 0; i < SIG_WORDS; i++)
     gdb_printf ("%08x ",
-		(unsigned int) bfd_get_32 (core_bfd, sigset + i * 4));
+		(unsigned int) bfd_get_32 (cbfd, sigset + i * 4));
   gdb_printf ("\n");
 }
 
@@ -1288,8 +1292,9 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
   size_t note_size;
   ULONGEST value;
   LONGEST sec;
+  bfd *cbfd = current_program_space->core_bfd ();
 
-  section = bfd_get_section_by_name (core_bfd, ".note.freebsdcore.proc");
+  section = bfd_get_section_by_name (cbfd, ".note.freebsdcore.proc");
   if (section == NULL)
     {
       warning (_("unable to find process info in core file"));
@@ -1299,7 +1304,7 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
   addr_bit = gdbarch_addr_bit (gdbarch);
   if (addr_bit == 64)
     kp = &kinfo_proc_layout_64;
-  else if (bfd_get_arch (core_bfd) == bfd_arch_i386)
+  else if (bfd_get_arch (cbfd) == bfd_arch_i386)
     kp = &kinfo_proc_layout_i386;
   else
     kp = &kinfo_proc_layout_32;
@@ -1317,8 +1322,7 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
     error (_("malformed core note - too short"));
 
   gdb::def_vector<unsigned char> contents (note_size);
-  if (!bfd_get_section_contents (core_bfd, section, contents.data (),
-				 0, note_size))
+  if (!bfd_get_section_contents (cbfd, section, contents.data (), 0, note_size))
     error (_("could not get core note contents"));
 
   descdata = contents.data ();
@@ -1327,7 +1331,7 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
   descdata += 4;
 
   /* Verify 'ki_layout' is 0.  */
-  if (bfd_get_32 (core_bfd, descdata + kp->ki_layout) != 0)
+  if (bfd_get_32 (cbfd, descdata + kp->ki_layout) != 0)
     {
       warning (_("unsupported process information in core file"));
       return;
@@ -1335,13 +1339,13 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
 
   gdb_printf ("Name: %.19s\n", descdata + kp->ki_comm);
   gdb_printf ("Process ID: %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_pid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_pid)));
   gdb_printf ("Parent process: %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_ppid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_ppid)));
   gdb_printf ("Process group: %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_pgid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_pgid)));
   gdb_printf ("Session id: %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_sid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_sid)));
 
   /* FreeBSD 12.0 and later store a 64-bit dev_t at 'ki_tdev'.  Older
      kernels store a 32-bit dev_t at 'ki_tdev_freebsd11'.  In older
@@ -1349,38 +1353,38 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
      the structure that is cleared to zero.  Assume that a zero value
      in ki_tdev indicates a core dump from an older kernel and use the
      value in 'ki_tdev_freebsd11' instead.  */
-  value = bfd_get_64 (core_bfd, descdata + kp->ki_tdev);
+  value = bfd_get_64 (cbfd, descdata + kp->ki_tdev);
   if (value == 0)
-    value = bfd_get_32 (core_bfd, descdata + kp->ki_tdev_freebsd11);
+    value = bfd_get_32 (cbfd, descdata + kp->ki_tdev_freebsd11);
   gdb_printf ("TTY: %s\n", pulongest (value));
   gdb_printf ("TTY owner process group: %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_tpgid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_tpgid)));
   gdb_printf ("User IDs (real, effective, saved): %s %s %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_ruid)),
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_uid)),
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_svuid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_ruid)),
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_uid)),
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_svuid)));
   gdb_printf ("Group IDs (real, effective, saved): %s %s %s\n",
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_rgid)),
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_groups)),
-	      pulongest (bfd_get_32 (core_bfd, descdata + kp->ki_svgid)));
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_rgid)),
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_groups)),
+	      pulongest (bfd_get_32 (cbfd, descdata + kp->ki_svgid)));
   gdb_printf ("Groups: ");
-  uint16_t ngroups = bfd_get_16 (core_bfd, descdata + kp->ki_ngroups);
+  uint16_t ngroups = bfd_get_16 (cbfd, descdata + kp->ki_ngroups);
   for (int i = 0; i < ngroups; i++)
     gdb_printf ("%s ",
-		pulongest (bfd_get_32 (core_bfd,
+		pulongest (bfd_get_32 (cbfd,
 				       descdata + kp->ki_groups + i * 4)));
   gdb_printf ("\n");
-  value = bfd_get (long_bit, core_bfd,
+  value = bfd_get (long_bit, cbfd,
 		   descdata + kp->ki_rusage + kp->ru_minflt);
   gdb_printf ("Minor faults (no memory page): %s\n", pulongest (value));
-  value = bfd_get (long_bit, core_bfd,
+  value = bfd_get (long_bit, cbfd,
 		   descdata + kp->ki_rusage_ch + kp->ru_minflt);
   gdb_printf ("Minor faults, children: %s\n", pulongest (value));
-  value = bfd_get (long_bit, core_bfd,
+  value = bfd_get (long_bit, cbfd,
 		   descdata + kp->ki_rusage + kp->ru_majflt);
   gdb_printf ("Major faults (memory page faults): %s\n",
 	      pulongest (value));
-  value = bfd_get (long_bit, core_bfd,
+  value = bfd_get (long_bit, cbfd,
 		   descdata + kp->ki_rusage_ch + kp->ru_majflt);
   gdb_printf ("Major faults, children: %s\n", pulongest (value));
   fbsd_core_fetch_timeval (gdbarch,
@@ -1404,22 +1408,22 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
   fbsd_core_fetch_timeval (gdbarch, descdata + kp->ki_start, sec, value);
   gdb_printf ("Start time: %s.%06d\n", plongest (sec), (int) value);
   gdb_printf ("Virtual memory size: %s kB\n",
-	      pulongest (bfd_get (addr_bit, core_bfd,
+	      pulongest (bfd_get (addr_bit, cbfd,
 				  descdata + kp->ki_size) / 1024));
   gdb_printf ("Data size: %s pages\n",
-	      pulongest (bfd_get (addr_bit, core_bfd,
+	      pulongest (bfd_get (addr_bit, cbfd,
 				  descdata + kp->ki_dsize)));
   gdb_printf ("Stack size: %s pages\n",
-	      pulongest (bfd_get (addr_bit, core_bfd,
+	      pulongest (bfd_get (addr_bit, cbfd,
 				  descdata + kp->ki_ssize)));
   gdb_printf ("Text size: %s pages\n",
-	      pulongest (bfd_get (addr_bit, core_bfd,
+	      pulongest (bfd_get (addr_bit, cbfd,
 				  descdata + kp->ki_tsize)));
   gdb_printf ("Resident set size: %s pages\n",
-	      pulongest (bfd_get (addr_bit, core_bfd,
+	      pulongest (bfd_get (addr_bit, cbfd,
 				  descdata + kp->ki_rssize)));
   gdb_printf ("Maximum RSS: %s pages\n",
-	      pulongest (bfd_get (long_bit, core_bfd,
+	      pulongest (bfd_get (long_bit, cbfd,
 				  descdata + kp->ki_rusage
 				  + kp->ru_maxrss)));
   fbsd_print_sigset ("Ignored Signals", descdata + kp->ki_sigignore);
@@ -1429,8 +1433,8 @@ fbsd_core_info_proc_status (struct gdbarch *gdbarch)
 /* Implement the "core_info_proc" gdbarch method.  */
 
 static void
-fbsd_core_info_proc (struct gdbarch *gdbarch, const char *args,
-		     enum info_proc_what what)
+fbsd_core_info_proc (struct gdbarch *gdbarch, struct bfd *cbfd,
+		     const char *args, enum info_proc_what what)
 {
   bool do_cmdline = false;
   bool do_cwd = false;
@@ -1478,7 +1482,7 @@ fbsd_core_info_proc (struct gdbarch *gdbarch, const char *args,
       return;
     }
 
-  pid = bfd_core_file_pid (core_bfd);
+  pid = bfd_core_file_pid (cbfd);
   if (pid != 0)
     gdb_printf (_("process %d\n"), pid);
 
@@ -1486,7 +1490,7 @@ fbsd_core_info_proc (struct gdbarch *gdbarch, const char *args,
     {
       const char *cmdline;
 
-      cmdline = bfd_core_file_failing_command (core_bfd);
+      cmdline = bfd_core_file_failing_command (cbfd);
       if (cmdline)
 	gdb_printf ("cmdline = '%s'\n", cmdline);
       else
@@ -1594,11 +1598,12 @@ fbsd_get_siginfo_type (struct gdbarch *gdbarch)
   if (fbsd_gdbarch_data->siginfo_type != NULL)
     return fbsd_gdbarch_data->siginfo_type;
 
-  int_type = arch_integer_type (gdbarch, gdbarch_int_bit (gdbarch),
+  type_allocator alloc (gdbarch);
+  int_type = init_integer_type (alloc, gdbarch_int_bit (gdbarch),
 				0, "int");
-  int32_type = arch_integer_type (gdbarch, 32, 0, "int32_t");
-  uint32_type = arch_integer_type (gdbarch, 32, 1, "uint32_t");
-  long_type = arch_integer_type (gdbarch, gdbarch_long_bit (gdbarch),
+  int32_type = init_integer_type (alloc, 32, 0, "int32_t");
+  uint32_type = init_integer_type (alloc, 32, 1, "uint32_t");
+  long_type = init_integer_type (alloc, gdbarch_long_bit (gdbarch),
 				 0, "long");
   void_ptr_type = lookup_pointer_type (builtin_type (gdbarch)->builtin_void);
 
@@ -1609,15 +1614,16 @@ fbsd_get_siginfo_type (struct gdbarch *gdbarch)
   append_composite_type_field (sigval_type, "sival_ptr", void_ptr_type);
 
   /* __pid_t */
-  pid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
-			int32_type->length () * TARGET_CHAR_BIT, "__pid_t");
+  pid_type = alloc.new_type (TYPE_CODE_TYPEDEF,
+			     int32_type->length () * TARGET_CHAR_BIT,
+			     "__pid_t");
   pid_type->set_target_type (int32_type);
   pid_type->set_target_is_stub (true);
 
   /* __uid_t */
-  uid_type = arch_type (gdbarch, TYPE_CODE_TYPEDEF,
-			uint32_type->length () * TARGET_CHAR_BIT,
-			"__uid_t");
+  uid_type = alloc.new_type (TYPE_CODE_TYPEDEF,
+			     uint32_type->length () * TARGET_CHAR_BIT,
+			     "__uid_t");
   uid_type->set_target_type (uint32_type);
   pid_type->set_target_is_stub (true);
 
@@ -1933,7 +1939,8 @@ fbsd_get_syscall_number (struct gdbarch *gdbarch, thread_info *thread)
 static LONGEST
 fbsd_read_integer_by_name (struct gdbarch *gdbarch, const char *name)
 {
-  bound_minimal_symbol ms = lookup_minimal_symbol (name, NULL, NULL);
+  bound_minimal_symbol ms
+    = lookup_minimal_symbol (current_program_space, name);
   if (ms.minsym == NULL)
     error (_("Unable to resolve symbol '%s'"), name);
 
@@ -1954,8 +1961,9 @@ fbsd_fetch_rtld_offsets (struct gdbarch *gdbarch, struct fbsd_pspace_data *data)
     {
       /* Fetch offsets from debug symbols in rtld.  */
       struct symbol *obj_entry_sym
-	= lookup_symbol_in_language ("Struct_Obj_Entry", NULL, STRUCT_DOMAIN,
-				     language_c, NULL).symbol;
+	= lookup_symbol_in_language ("Struct_Obj_Entry", nullptr,
+				     SEARCH_STRUCT_DOMAIN,
+				     language_c, nullptr).symbol;
       if (obj_entry_sym == NULL)
 	error (_("Unable to find Struct_Obj_Entry symbol"));
       data->off_linkmap = lookup_struct_elt (obj_entry_sym->type (),
@@ -2023,21 +2031,23 @@ fbsd_get_thread_local_address (struct gdbarch *gdbarch, CORE_ADDR dtv_addr,
 {
   LONGEST tls_index = fbsd_get_tls_index (gdbarch, lm_addr);
 
-  gdb_byte buf[gdbarch_ptr_bit (gdbarch) / TARGET_CHAR_BIT];
-  if (target_read_memory (dtv_addr, buf, sizeof buf) != 0)
+  gdb::byte_vector buf (gdbarch_ptr_bit (gdbarch) / TARGET_CHAR_BIT);
+  if (target_read_memory (dtv_addr, buf.data (), buf.size ()) != 0)
     throw_error (TLS_GENERIC_ERROR,
 		 _("Cannot find thread-local variables on this target"));
 
   const struct builtin_type *builtin = builtin_type (gdbarch);
-  CORE_ADDR addr = gdbarch_pointer_to_address (gdbarch,
-					       builtin->builtin_data_ptr, buf);
+  CORE_ADDR addr
+    = gdbarch_pointer_to_address (gdbarch, builtin->builtin_data_ptr,
+				  buf.data ());
 
   addr += (tls_index + 1) * builtin->builtin_data_ptr->length ();
-  if (target_read_memory (addr, buf, sizeof buf) != 0)
+  if (target_read_memory (addr, buf.data (), buf.size ()) != 0)
     throw_error (TLS_GENERIC_ERROR,
 		 _("Cannot find thread-local variables on this target"));
 
-  addr = gdbarch_pointer_to_address (gdbarch, builtin->builtin_data_ptr, buf);
+  addr = gdbarch_pointer_to_address (gdbarch, builtin->builtin_data_ptr,
+				     buf.data ());
   return addr + offset;
 }
 
@@ -2046,7 +2056,8 @@ fbsd_get_thread_local_address (struct gdbarch *gdbarch, CORE_ADDR dtv_addr,
 CORE_ADDR
 fbsd_skip_solib_resolver (struct gdbarch *gdbarch, CORE_ADDR pc)
 {
-  struct bound_minimal_symbol msym = lookup_bound_minimal_symbol ("_rtld_bind");
+  bound_minimal_symbol msym
+    = lookup_minimal_symbol (current_program_space, "_rtld_bind");
   if (msym.minsym != nullptr && msym.value_address () == pc)
     return frame_unwind_caller_pc (get_current_frame ());
 
@@ -2314,7 +2325,8 @@ fbsd_vdso_range (struct gdbarch *gdbarch, struct mem_range *range)
   if (!target_has_execution ())
     {
       /* Search for the ending address in the NT_PROCSTAT_VMMAP note. */
-      asection *section = bfd_get_section_by_name (core_bfd,
+      bfd *cbfd = current_program_space->core_bfd ();
+      asection *section = bfd_get_section_by_name (cbfd,
 						   ".note.freebsdcore.vmmap");
       if (section == nullptr)
 	return false;
@@ -2324,7 +2336,7 @@ fbsd_vdso_range (struct gdbarch *gdbarch, struct mem_range *range)
 	return false;
 
       gdb::def_vector<unsigned char> contents (note_size);
-      if (!bfd_get_section_contents (core_bfd, section, contents.data (),
+      if (!bfd_get_section_contents (cbfd, section, contents.data (),
 				     0, note_size))
 	return false;
 
@@ -2334,7 +2346,7 @@ fbsd_vdso_range (struct gdbarch *gdbarch, struct mem_range *range)
   else
     {
       /* Fetch the list of address space entries from the running target. */
-      gdb::optional<gdb::byte_vector> buf =
+      std::optional<gdb::byte_vector> buf =
 	target_read_alloc (current_inferior ()->top_target (),
 			   TARGET_OBJECT_FREEBSD_VMMAP, nullptr);
       if (!buf || buf->empty ())
@@ -2346,6 +2358,137 @@ fbsd_vdso_range (struct gdbarch *gdbarch, struct mem_range *range)
   return range->length != 0;
 }
 
+/* Try to extract the inferior arguments, environment, and executable name
+   from CBFD.  */
+
+static core_file_exec_context
+fbsd_corefile_parse_exec_context_1 (struct gdbarch *gdbarch, bfd *cbfd)
+{
+  gdb_assert (gdbarch != nullptr);
+
+  /* If there's no core file loaded then we're done.  */
+  if (cbfd == nullptr)
+    return {};
+
+  int ptr_bytes = gdbarch_ptr_bit (gdbarch) / TARGET_CHAR_BIT;
+
+  /* Find the .auxv section in the core file. The BFD library creates this
+     for us from the AUXV note when the BFD is opened.  If the section
+     can't be found then there's nothing more we can do.  */
+  struct bfd_section * section = bfd_get_section_by_name (cbfd, ".auxv");
+  if (section == nullptr)
+    return {};
+
+  /* Grab the contents of the .auxv section.  If we can't get the contents
+     then there's nothing more we can do.  */
+  bfd_size_type size = bfd_section_size (section);
+  if (bfd_section_size_insane (cbfd, section))
+    return {};
+  gdb::byte_vector contents (size);
+  if (!bfd_get_section_contents (cbfd, section, contents.data (), 0, size))
+    return {};
+
+  /* Read AT_FREEBSD_ARGV, the address of the argument string vector.  */
+  CORE_ADDR argv_addr;
+  if (target_auxv_search (contents, current_inferior ()->top_target (),
+			  gdbarch, AT_FREEBSD_ARGV, &argv_addr) != 1)
+    return {};
+
+  /* Read AT_FREEBSD_ARGV, the address of the environment string vector.  */
+  CORE_ADDR envv_addr;
+  if (target_auxv_search (contents, current_inferior ()->top_target (),
+			  gdbarch, AT_FREEBSD_ENVV, &envv_addr) != 1)
+    return {};
+
+  /* Read the AT_EXECPATH string.  It's OK if we can't get this
+     information.  */
+  gdb::unique_xmalloc_ptr<char> execpath;
+  CORE_ADDR execpath_string_addr;
+  if (target_auxv_search (contents, current_inferior ()->top_target (),
+			  gdbarch, AT_FREEBSD_EXECPATH,
+			  &execpath_string_addr) == 1)
+    execpath = target_read_string (execpath_string_addr, INT_MAX);
+
+  /* The byte order.  */
+  enum bfd_endian byte_order = gdbarch_byte_order (gdbarch);
+
+  /* On FreeBSD the command the user ran is found in argv[0].  When we
+     read the first argument we place it into EXECFN.  */
+  gdb::unique_xmalloc_ptr<char> execfn;
+
+  /* Read strings from AT_FREEBSD_ARGV until we find a NULL marker.  The
+     first argument is placed into EXECFN as the command name.  */
+  std::vector<gdb::unique_xmalloc_ptr<char>> arguments;
+  CORE_ADDR str_addr;
+  while ((str_addr
+	  = (CORE_ADDR) read_memory_unsigned_integer (argv_addr, ptr_bytes,
+						      byte_order)) != 0)
+    {
+      gdb::unique_xmalloc_ptr<char> str
+	= target_read_string (str_addr, INT_MAX);
+      if (str == nullptr)
+	return {};
+
+      if (execfn == nullptr)
+	execfn = std::move (str);
+      else
+	arguments.emplace_back (std::move (str));
+
+      argv_addr += ptr_bytes;
+    }
+
+  /* Read strings from AT_FREEBSD_ENVV until we find a NULL marker.  */
+  std::vector<gdb::unique_xmalloc_ptr<char>> environment;
+  while ((str_addr
+	  = (uint64_t) read_memory_unsigned_integer (envv_addr, ptr_bytes,
+						     byte_order)) != 0)
+    {
+      gdb::unique_xmalloc_ptr<char> str
+	= target_read_string (str_addr, INT_MAX);
+      if (str == nullptr)
+	return {};
+
+      environment.emplace_back (std::move (str));
+      envv_addr += ptr_bytes;
+    }
+
+  return core_file_exec_context (std::move (execfn),
+				 std::move (execpath),
+				 std::move (arguments),
+				 std::move (environment));
+}
+
+/* See elf-corelow.h.  */
+
+static core_file_exec_context
+fbsd_corefile_parse_exec_context (struct gdbarch *gdbarch, bfd *cbfd)
+{
+  /* Catch and discard memory errors.
+
+     If the core file format is not as we expect then we can easily trigger
+     a memory error while parsing the core file.  We don't want this to
+     prevent the user from opening the core file; the information provided
+     by this function is helpful, but not critical, debugging can continue
+     without it.  Instead just give a warning and return an empty context
+     object.  */
+  try
+    {
+      return fbsd_corefile_parse_exec_context_1 (gdbarch, cbfd);
+    }
+  catch (const gdb_exception_error &ex)
+    {
+      if (ex.error == MEMORY_ERROR)
+	{
+	  warning
+	    (_("failed to parse execution context from corefile: %s"),
+	     ex.message->c_str ());
+	  return {};
+	}
+      else
+	throw;
+    }
+}
+
 /* Return the address range of the vDSO for the current inferior.  */
 
 static int
@@ -2389,4 +2532,6 @@ fbsd_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   /* `catch syscall' */
   set_xml_syscall_file_name (gdbarch, "syscalls/freebsd.xml");
   set_gdbarch_get_syscall_number (gdbarch, fbsd_get_syscall_number);
+  set_gdbarch_core_parse_exec_context (gdbarch,
+				       fbsd_corefile_parse_exec_context);
 }
